﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Collections.Concurrent;
using System.Threading;
using System.Threading.Tasks;

namespace CustomProducerConsumer
{
    class ConcurrentAggregate<T>: IProducerConsumerCollection<T>
    {
        Dictionary<T, int> collection =new Dictionary<T,int>();

        public void Add(T item)
        {
            int count;
            if (collection.TryGetValue(item, out count))
            {
                collection[item] = count + 1;
            }
            else
            {
                collection.Add(item, 1);
            }
        }

        public bool Remove(T item)
        {

            int count;
            if (collection.TryGetValue(item, out count))
            {
                --count;
                if (count > 0)
                {
                    collection[item] = count;
                    return true;
                }
                else
                {
                    return collection.Remove(item);
                }
            }
            return false;
        }

        public int GetInstanceCount(T key)
        {
            lock (this)
            {
                return collection[key];
            }
        }
        
        public IEnumerator<T> GetEnumerator()
        {
            return ToArray().AsEnumerable().GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return ToArray().AsEnumerable().GetEnumerator();
        }

        public bool TryAdd(T item)
        {
            lock (this)
            {
                Add(item);
            }

            return true;
        }

        public bool TryTake(out T item)
        {
            lock (this)
            {
                item = collection.First().Key;
                collection.Remove(item);
                return true;
            }
        }

        public T[] ToArray()
        {
            List<T> temp=new List<T>();
            foreach (var item in collection)
            {
                temp.Add(item.Key);
            }
            return temp.ToArray();
        }

        public void CopyTo(T[] array, int index)
        {
            lock (this)
            {
                ToArray().CopyTo(array, index);
            }           
        }

        public void CopyTo(Array array, int index)
        {
            lock (this)
            {
               ToArray().CopyTo(array, index);
            }
        }

        public bool IsSynchronized { get { return false; } }
        public int Count { get { return collection.Count(); } }
        public Object SyncRoot { get { throw new NotSupportedException(); } }
    }

    
    class Program
    {
        static void Main(string[] args)
        {
            var list = new ConcurrentAggregate<int>();
            var numbers = new int[] { 4, 2, 1, 3, 3, 3, 2, 4, 1, 5 };
            Parallel.ForEach(numbers, (number) =>
            {
                list.TryAdd(number);
            });

            foreach (var item in list)
            {
                Console.WriteLine("Item {0} Instances {1}", 
                    item, list.GetInstanceCount(item)); 
            }
            Console.WriteLine("Press enter to exit");
            Console.ReadLine();
        }
    }
}
