﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Concurrent;
using System.Threading;
using System.Collections;
using System.Threading.Tasks;

namespace PartitionerNamespace
{

    class CustomPartitioner<TSource>: Partitioner<TSource>
    {
        IList<TSource> list;

        public CustomPartitioner(IList<TSource> input)
        {
            list = input;
        }

        class EnumerableWrapper : IEnumerable<TSource>
        {
            IList<TSource> list = null;
            int index = -1;
            int partitionSize = 0;

            public EnumerableWrapper(IList<TSource> _list)
            {
                list = _list;
            }

            public IEnumerator<TSource> GetEnumerator()
            {
                int localPartition = Interlocked.Increment(ref partitionSize);
                int localIndex = Interlocked.Increment(ref index);
                int begin = localIndex;
                int end = begin + localPartition;
                int i = 0;
                for (i = begin; i < end; ++i)
                {
                    if (i >= list.Count)
                    {
                        localIndex = i;
                        yield break;
                    }
                    yield return list[i];
                }
                Interlocked.Decrement(ref index);        
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                return ((IEnumerable<TSource>)this).GetEnumerator();      
            }
        }

        public override IEnumerable<TSource> GetDynamicPartitions()
        {
            return new EnumerableWrapper(list);
        }

        public override bool SupportsDynamicPartitions { get { return true; } }

        public override IList<IEnumerator<TSource>> GetPartitions(int partitionCount)
        {
            if (partitionCount < 1)
                throw new ArgumentOutOfRangeException("partitionCount");
            var obj = new List<IEnumerator<TSource>>();
            var enumerable = GetDynamicPartitions();
            for (int i = 0; i < partitionCount; ++i)
            {
                obj.Add(enumerable.GetEnumerator());
            };
            return obj;
        } 
    }

    class Program
    {

        static void Main(string[] args)
        {
            int [] numbers = new int[] { 1, 2, 3, 5, 7, 8, 4, 5, 6, 7, 8, 9, 10 };
            var samplePartitioner = new CustomPartitioner<int>(numbers);
            Parallel.ForEach(samplePartitioner, (item) =>
            {
                Thread.SpinWait(10000);
                Console.WriteLine("Task {0} Item {1}", Task.CurrentId, item);
            });

            Console.WriteLine("Press enter to exit");
            Console.ReadLine();
        }
    }
}
