﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Collections.Concurrent;

namespace Scheduler
{
    public class CustomScheduler : TaskScheduler
    {
        List<Task> tasks = new List<Task>();
        Action<Task> Setup;
        Action<Task> Cleanup;

        public CustomScheduler(Action<Task> _Setup=null,
            Action<Task> _Cleanup=null)
        {
            Setup = _Setup;
            Cleanup = _Cleanup;
        }

        protected override void QueueTask(Task currentTask)
        {
            lock (this)
            {
                tasks.Add(currentTask);
            }
            Thread t = new Thread(new ThreadStart(() =>
            {
                if(Setup!=null) Setup(currentTask);
                TryExecuteTask(currentTask); 
                if(Cleanup!=null) Cleanup(currentTask);
            }));
            t.IsBackground = true;
            t.Start();
        }

        protected override bool TryDequeue(Task task)
        {
            lock (this)
            {
                tasks.Remove(task);
            }
            return true;
        }

        protected override IEnumerable<Task> GetScheduledTasks()
        {
            return tasks.ToArray();
        }

        protected override bool TryExecuteTaskInline(Task task, bool taskWasPreviouslyQueued)
        {
            return false;
        }
    }
    class Program
    {
        static void Main(string[] args)
        {
            var numbers = Enumerable.Range(0, 10);
            ParallelOptions options = new ParallelOptions();
            options.TaskScheduler = new CustomScheduler((task) =>
            {
                Console.WriteLine("Before Task {0,2}", task.Id);
            }, (task) =>
            {
                Console.WriteLine("After  Task {0,2}", task.Id);
            });

            Parallel.ForEach(numbers, options, (number) =>
            {
                Console.WriteLine("\t\tValue = {0}", number);
            });

            //Console.WriteLine("Press enter to exit");
            Console.ReadLine();

        }
    }
}
